from setuptools import setup

setup(
    name='qlang',
    version='0.1.0',
    author='nirvanasupermind',
    author_email='nirvana.supermind@gmail.com',
    url='https://github.com/nirvanasupermind/qlang',
    license='LICENSE',
    packages=['.'],
    description='A concise, procedural programming language',
    install_requires=[]
)